@@ -2,11 +2,9 @@
  *
  * Giada - Your Hardcore Loopmachine
  *
- * const.h
- *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -27,18 +25,18 @@
  * -------------------------------------------------------------------------- */
 
 
-#ifndef CONST_H
-#define CONST_H
+#ifndef G_CONST_H
+#define G_CONST_H
 
 
 /* -- version --------------------------------------------------------------- */
-#define G_VERSION_STR   "0.13.2"
 #define G_APP_NAME      "Giada"
+#define G_VERSION_STR   "0.14.0"
 #define G_VERSION_MAJOR 0
-#define G_VERSION_MINOR 13
-#define G_VERSION_PATCH 2
+#define G_VERSION_MINOR 14
+#define G_VERSION_PATCH 0
 
-#define CONF_FILENAME		"giada.conf"
+#define CONF_FILENAME "giada.conf"
 
 #ifndef BUILD_DATE
 	#define BUILD_DATE __DATE__
@@ -82,67 +80,68 @@
 
 
 /* -- MIN/MAX values -------------------------------------------------------- */
-#define MAX_BEATS				   32
-#define MAX_BARS				   32
-#define MAX_PATCHNAME_LEN	 32
-#define DB_MIN_SCALE		   60.0f
-#define MIN_COLUMN_WIDTH   140
+#define G_MIN_BPM           20.0f
+#define G_MAX_BPM           999.0f
+#define G_MAX_BEATS				  32
+#define G_MAX_BARS				  32
+#define G_MAX_QUANTIZE      8
+#define G_MAX_PATCHNAME_LEN	32
+#define G_DB_MIN_SCALE      60.0f
+#define G_MIN_COLUMN_WIDTH  140
+#define G_MAX_BOOST_DB      20.0f
+#define G_MAX_PITCH         4.0f
 
 
 
 /* -- kernel audio ---------------------------------------------------------- */
-#define SYS_API_NONE		0x00  // 0000 0000
-#define SYS_API_JACK		0x01  // 0000 0001
-#define SYS_API_ALSA		0x02  // 0000 0010
-#define SYS_API_DS			0x04  // 0000 0100
-#define SYS_API_ASIO		0x08  // 0000 1000
-#define SYS_API_CORE		0x10  // 0001 0000
-#define SYS_API_PULSE   0x20  // 0010 0000
-#define SYS_API_WASAPI  0x40  // 0100 0000
-#define SYS_API_ANY     0x7F  // 0111 1111
-
-#define KERNEL_OK					 0
-#define KERNEL_UNDERRUN	  -1
-#define KERNEL_CRITICAL	  -2
+#define G_SYS_API_NONE		0x00  // 0000 0000
+#define G_SYS_API_JACK		0x01  // 0000 0001
+#define G_SYS_API_ALSA		0x02  // 0000 0010
+#define G_SYS_API_DS			0x04  // 0000 0100
+#define G_SYS_API_ASIO		0x08  // 0000 1000
+#define G_SYS_API_CORE		0x10  // 0001 0000
+#define G_SYS_API_PULSE   0x20  // 0010 0000
+#define G_SYS_API_WASAPI  0x40  // 0100 0000
+#define G_SYS_API_ANY     0x7F  // 0111 1111
 
 
 
 /* -- kernel midi ----------------------------------------------------------- */
-#define MIDI_API_JACK		0x01  // 0000 0001
-#define MIDI_API_ALSA		0x02  // 0000 0010
+#define G_MIDI_API_JACK		0x01  // 0000 0001
+#define G_MIDI_API_ALSA		0x02  // 0000 0010
 
 
 
 /* -- default system -------------------------------------------------------- */
 #if defined(__linux__)
-	#define DEFAULT_SOUNDSYS	SYS_API_NONE
+	#define G_DEFAULT_SOUNDSYS	G_SYS_API_NONE
 #elif defined(_WIN32)
-	#define DEFAULT_SOUNDSYS 	SYS_API_DS
+	#define G_DEFAULT_SOUNDSYS 	G_SYS_API_DS
 #elif defined(__APPLE__)
-	#define DEFAULT_SOUNDSYS 	SYS_API_CORE
+	#define G_DEFAULT_SOUNDSYS 	G_SYS_API_CORE
 #endif
 
-#define DEFAULT_SOUNDDEV_OUT  0       /// FIXME - please override with rtAudio::getDefaultDevice (or similar)
-#define DEFAULT_SOUNDDEV_IN   -1			// no recording by default: input disabled
-#define DEFAULT_MIDI_SYSTEM   0
-#define DEFAULT_MIDI_PORT_IN  -1
-#define DEFAULT_MIDI_PORT_OUT -1
-#define DEFAULT_SAMPLERATE   44100
-#define DEFAULT_BUFSIZE		   1024
-#define DEFAULT_DELAYCOMP		 0
-#define DEFAULT_VOL				   1.0f
-#define G_DEFAULT_PITCH			 1.0f
-#define DEFAULT_BOOST			   0.0f
-#define DEFAULT_OUT_VOL  	   1.0f
-#define DEFAULT_IN_VOL  	   1.0f
-#define DEFAULT_CHANMODE	   SINGLE_BASIC
-#define DEFAULT_BPM				   120.0f
-#define DEFAULT_BEATS			   4
-#define DEFAULT_BARS			   1
-#define DEFAULT_QUANTIZE     0		  // quantizer off
-#define DEFAULT_FADEOUT_STEP 0.01f  // micro-fadeout speed
-#define DEFAULT_COLUMN_WIDTH 380
-#define G_DEFAULT_PATCH_NAME "(default patch)"
+#define G_DEFAULT_SOUNDDEV_OUT    0      // FIXME - please override with rtAudio::getDefaultDevice (or similar)
+#define G_DEFAULT_SOUNDDEV_IN    -1			// no recording by default: input disabled
+#define G_DEFAULT_MIDI_SYSTEM     0
+#define G_DEFAULT_MIDI_PORT_IN   -1
+#define G_DEFAULT_MIDI_PORT_OUT  -1
+#define G_DEFAULT_SAMPLERATE      44100
+#define G_DEFAULT_BUFSIZE		      1024
+#define G_DEFAULT_DELAYCOMP		    0
+#define G_DEFAULT_VOL				      1.0f
+#define G_DEFAULT_PITCH			      1.0f
+#define G_DEFAULT_BOOST			      1.0f
+#define G_DEFAULT_OUT_VOL  	      1.0f
+#define G_DEFAULT_IN_VOL  	      1.0f
+#define G_DEFAULT_CHANMODE	      SINGLE_BASIC
+#define G_DEFAULT_BPM				      120.0f
+#define G_DEFAULT_BEATS			      4
+#define G_DEFAULT_BARS			      1
+#define G_DEFAULT_QUANTIZE        0		  // quantizer off
+#define G_DEFAULT_FADEOUT_STEP    0.01f  // micro-fadeout speed
+#define G_DEFAULT_COLUMN_WIDTH    380
+#define G_DEFAULT_PATCH_NAME       "(default patch)"
 #define G_DEFAULT_MIDI_INPUT_UI_W 300
 #define G_DEFAULT_MIDI_INPUT_UI_H 350
 
@@ -177,19 +176,19 @@
 
 
 /* -- actions --------------------------------------------------------------- */
-#define ACTION_KEYPRESS		0x01 // 0000 0001
-#define ACTION_KEYREL			0x02 // 0000 0010
-#define ACTION_KILLCHAN		0x04 // 0000 0100
-#define ACTION_MUTEON			0x08 // 0000 1000
-#define ACTION_MUTEOFF		0x10 // 0001 0000
-#define ACTION_VOLUME     0x20 // 0010 0000
-#define ACTION_MIDI       0x40 // 0100 0000
+#define G_ACTION_KEYPRESS		0x01 // 0000 0001
+#define G_ACTION_KEYREL			0x02 // 0000 0010
+#define G_ACTION_KILL		    0x04 // 0000 0100
+#define G_ACTION_MUTEON			0x08 // 0000 1000
+#define G_ACTION_MUTEOFF		0x10 // 0001 0000
+#define G_ACTION_VOLUME     0x20 // 0010 0000
+#define G_ACTION_MIDI       0x40 // 0100 0000
 
-#define ACTION_KEYS       0x03 // 0000 0011 any key
-#define ACTION_MUTES      0x24 // 0001 1000 any mute
+#define G_ACTION_KEYS       0x03 // 0000 0011 any key
+#define G_ACTION_MUTES      0x24 // 0001 1000 any mute
 
-#define RANGE_CHAR        0x01 // range for MIDI (0-127)
-#define RANGE_FLOAT       0x02 // range for volumes and VST params (0.0-1.0)
+#define G_RANGE_CHAR        0x01 // range for MIDI (0-127)
+#define G_RANGE_FLOAT       0x02 // range for volumes and VST params (0.0-1.0)
 
 
 
@@ -254,11 +253,10 @@
 
 
 /* -- MIDI signals -------------------------------------------------------------
- * all signals are set to channel 0 (where channels are considered).
- * It's up to the caller to bitmask them with the proper channel number. */
-
-/* channel voices messages - controller (0xB0) is a special subset of
- * this family: it drives knobs, volume, faders and such. */
+All signals are set to channel 0 (where channels are considered). It's up to the
+caller to bitmask them with the proper channel number.
+Channel voices messages - controller (0xB0) is a special subset of this family:
+it drives knobs, volume, faders and such. */
 
 #define MIDI_CONTROLLER     0xB0 << 24
 #define MIDI_NOTE_ON        0x90 << 24
@@ -307,10 +305,10 @@ const int MIDI_CHANS[16] = {
 /* midi sync constants */
 
 #define MIDI_SYNC_NONE      0x00
-#define MIDI_SYNC_CLOCK_M   0x01
-#define MIDI_SYNC_CLOCK_S   0x02
-#define MIDI_SYNC_MTC_M     0x04
-#define MIDI_SYNC_MTC_S     0x08
+#define MIDI_SYNC_CLOCK_M   0x01  // master
+#define MIDI_SYNC_CLOCK_S   0x02  // slave
+#define MIDI_SYNC_MTC_M     0x04  // master
+#define MIDI_SYNC_MTC_S     0x08  // slave
 
 /* JSON patch keys */
 
@@ -340,8 +338,7 @@ const int MIDI_CHANS[16] = {
 #define PATCH_KEY_CHANNEL_MUTE_S               "mute_s"
 #define PATCH_KEY_CHANNEL_SOLO                 "solo"
 #define PATCH_KEY_CHANNEL_VOLUME               "volume"
-#define PATCH_KEY_CHANNEL_PAN_LEFT             "pan_left"
-#define PATCH_KEY_CHANNEL_PAN_RIGHT            "pan_right"
+#define PATCH_KEY_CHANNEL_PAN                  "pan"
 #define PATCH_KEY_CHANNEL_MIDI_IN              "midi_in"
 #define PATCH_KEY_CHANNEL_MIDI_IN_KEYPRESS     "midi_in_keypress"
 #define PATCH_KEY_CHANNEL_MIDI_IN_KEYREL       "midi_in_keyrel"
@@ -362,6 +359,7 @@ const int MIDI_CHANS[16] = {
 #define PATCH_KEY_CHANNEL_BOOST                "boost"
 #define PATCH_KEY_CHANNEL_REC_ACTIVE           "rec_active"
 #define PATCH_KEY_CHANNEL_PITCH                "pitch"
+#define PATCH_KEY_CHANNEL_INPUT_MONITOR        "input_monitor"
 #define PATCH_KEY_CHANNEL_MIDI_IN_READ_ACTIONS "midi_in_read_actions"
 #define PATCH_KEY_CHANNEL_MIDI_IN_PITCH        "midi_in_pitch"
 #define PATCH_KEY_CHANNEL_MIDI_OUT             "midi_out"
@@ -415,6 +413,7 @@ const int MIDI_CHANS[16] = {
 #define CONF_KEY_CHANS_STOP_ON_SEQ_HALT   "chans_stop_on_seq_halt"
 #define CONF_KEY_TREAT_RECS_AS_LOOPS      "treat_recs_as_loops"
 #define CONF_KEY_RESIZE_RECORDINGS        "resize_recordings"
+#define CONF_KEY_INPUT_MONITOR_DEFAULT_ON "input_monitor_default_on"
 #define CONF_KEY_PLUGINS_PATH             "plugins_path"
 #define CONF_KEY_PATCHES_PATH             "patches_path"
 #define CONF_KEY_SAMPLES_PATH             "samples_path"
