@@ -2,11 +2,9 @@
  *
  * Giada - Your Hardcore Loopmachine
  *
- * pluginHost
- *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -31,16 +29,71 @@
 
 
 #include "../utils/log.h"
+#include "../utils/fs.h"
 #include "const.h"
 #include "channel.h"
 #include "plugin.h"
 #include "pluginHost.h"
 
 
+using std::vector;
 using std::string;
 
 
-PluginHost::~PluginHost()
+namespace giada {
+namespace m {
+namespace pluginHost
+{
+namespace
+{
+juce::MessageManager *messageManager;
+
+/* pluginFormat
+ * Plugin format manager. */
+
+juce::VSTPluginFormat pluginFormat;
+
+/* knownPuginList
+ * List of known (i.e. scanned) plugins. */
+
+juce::KnownPluginList knownPluginList;
+
+/* unknownPluginList
+ * List of unrecognized plugins found in a patch. */
+
+std::vector<std::string> unknownPluginList;
+
+std::vector<Plugin*> masterOut;
+std::vector<Plugin*> masterIn;
+
+/* Audio|MidiBuffer
+ * Dynamic buffers. */
+
+juce::AudioBuffer<float> audioBuffer;
+
+int samplerate;
+int buffersize;
+
+/* missingPlugins
+ * If some plugins from any stack are missing. */
+
+bool missingPlugins;
+
+}; // {anonymous}
+
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+
+pthread_mutex_t mutex_midi;
+
+
+/* -------------------------------------------------------------------------- */
+
+
+void close()
 {
   messageManager->deleteInstance();
 }
@@ -49,9 +102,9 @@ PluginHost::~PluginHost()
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::init(int _buffersize, int _samplerate)
+void init(int _buffersize, int _samplerate)
 {
-  gu_log("[PluginHost::init] initialize with buffersize=%d, samplerate=%d\n",
+  gu_log("[pluginHost::init] initialize with buffersize=%d, samplerate=%d\n",
     _buffersize, _samplerate);
 
   messageManager = juce::MessageManager::getInstance();
@@ -62,18 +115,18 @@ void PluginHost::init(int _buffersize, int _samplerate)
   //unknownPluginList.empty();
   loadList(gu_getHomePath() + G_SLASH + "plugins.xml");
 
-  pthread_mutex_init(&mutex_midi, NULL);
+  pthread_mutex_init(&mutex_midi, nullptr);
 }
 
 
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::scanDir(const string &dirpath, void (*callback)(float progress, void *p),
+int scanDir(const string &dirpath, void (*callback)(float progress, void *p),
     void *p)
 {
-  gu_log("[PluginHost::scanDir] requested directory: '%s'\n", dirpath.c_str());
-  gu_log("[PluginHost::scanDir] current plugins: %d\n", knownPluginList.getNumTypes());
+  gu_log("[pluginHost::scanDir] requested directory: '%s'\n", dirpath.c_str());
+  gu_log("[pluginHost::scanDir] current plugins: %d\n", knownPluginList.getNumTypes());
 
   knownPluginList.clear();   // clear up previous plugins
 
@@ -85,13 +138,13 @@ int PluginHost::scanDir(const string &dirpath, void (*callback)(float progress,
   bool cont = true;
   juce::String name;
   while (cont) {
-    gu_log("[PluginHost::scanDir]   scanning '%s'\n", name.toRawUTF8());
+    gu_log("[pluginHost::scanDir]   scanning '%s'\n", name.toRawUTF8());
     cont = scanner.scanNextFile(false, name);
     if (callback)
       callback(scanner.getProgress(), p);
   }
 
-  gu_log("[PluginHost::scanDir] %d plugin(s) found\n", knownPluginList.getNumTypes());
+  gu_log("[pluginHost::scanDir] %d plugin(s) found\n", knownPluginList.getNumTypes());
   return knownPluginList.getNumTypes();
 }
 
@@ -99,11 +152,11 @@ int PluginHost::scanDir(const string &dirpath, void (*callback)(float progress,
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::saveList(const string &filepath)
+int saveList(const string &filepath)
 {
   int out = knownPluginList.createXml()->writeToFile(juce::File(filepath), "");
   if (!out)
-    gu_log("[PluginHost::saveList] unable to save plugin list to %s\n", filepath.c_str());
+    gu_log("[pluginHost::saveList] unable to save plugin list to %s\n", filepath.c_str());
   return out;
 }
 
@@ -111,7 +164,7 @@ int PluginHost::saveList(const string &filepath)
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::loadList(const string &filepath)
+int loadList(const string &filepath)
 {
   juce::XmlElement *elem = juce::XmlDocument::parse(juce::File(filepath));
   if (elem) {
@@ -126,8 +179,8 @@ int PluginHost::loadList(const string &filepath)
 /* -------------------------------------------------------------------------- */
 
 
-Plugin *PluginHost::addPlugin(const string &fid, int stackType,
-  pthread_mutex_t *mutex, class Channel *ch)
+Plugin *addPlugin(const string &fid, int stackType,
+  pthread_mutex_t *mutex, Channel *ch)
 {
   /* Get the proper stack to add the plugin to */
 
@@ -138,19 +191,19 @@ Plugin *PluginHost::addPlugin(const string &fid, int stackType,
 
   juce::PluginDescription *pd = knownPluginList.getTypeForFile(fid);
   if (!pd) {
-    gu_log("[PluginHost::addPlugin] no plugin found with fid=%s!\n", fid.c_str());
+    gu_log("[pluginHost::addPlugin] no plugin found with fid=%s!\n", fid.c_str());
     missingPlugins = true;
     unknownPluginList.push_back(fid);
-    return NULL;
+    return nullptr;
   }
 
   juce::AudioPluginInstance *pi = pluginFormat.createInstanceFromDescription(*pd, samplerate, buffersize);
   if (!pi) {
-    gu_log("[PluginHost::addPlugin] unable to create instance with fid=%s!\n", fid.c_str());
+    gu_log("[pluginHost::addPlugin] unable to create instance with fid=%s!\n", fid.c_str());
     missingPlugins = true;
-    return NULL;
+    return nullptr;
   }
-  gu_log("[PluginHost::addPlugin] plugin instance with fid=%s created\n", fid.c_str());
+  gu_log("[pluginHost::addPlugin] plugin instance with fid=%s created\n", fid.c_str());
 
   Plugin *p = new Plugin(pi, samplerate, buffersize);
 
@@ -164,7 +217,7 @@ Plugin *PluginHost::addPlugin(const string &fid, int stackType,
     break;
   }
 
-  gu_log("[PluginHost::addPlugin] plugin id=%s loaded (%s), stack type=%d, stack size=%d\n",
+  gu_log("[pluginHost::addPlugin] plugin id=%s loaded (%s), stack type=%d, stack size=%d\n",
     fid.c_str(), p->getName().c_str(), stackType, pStack->size());
 
   return p;
@@ -174,18 +227,18 @@ Plugin *PluginHost::addPlugin(const string &fid, int stackType,
 /* -------------------------------------------------------------------------- */
 
 
-Plugin *PluginHost::addPlugin(int index, int stackType, pthread_mutex_t *mutex,
-  class Channel *ch)
+Plugin *addPlugin(int index, int stackType, pthread_mutex_t *mutex,
+  Channel *ch)
 {
   juce::PluginDescription *pd = knownPluginList.getType(index);
   if (pd) {
-    gu_log("[PluginHost::addPlugin] plugin found, uid=%s, name=%s...\n",
+    gu_log("[pluginHost::addPlugin] plugin found, uid=%s, name=%s...\n",
       pd->fileOrIdentifier.toStdString().c_str(), pd->name.toStdString().c_str());
     return addPlugin(pd->fileOrIdentifier.toStdString(), stackType, mutex, ch);
   }
   else {
-    gu_log("[PluginHost::addPlugin] no plugins found at index=%d!\n", index);
-    return NULL;
+    gu_log("[pluginHost::addPlugin] no plugins found at index=%d!\n", index);
+    return nullptr;
   }
 }
 
@@ -193,7 +246,7 @@ Plugin *PluginHost::addPlugin(int index, int stackType, pthread_mutex_t *mutex,
 /* -------------------------------------------------------------------------- */
 
 
-vector <Plugin *> *PluginHost::getStack(int stackType, Channel *ch)
+vector <Plugin *> *getStack(int stackType, Channel *ch)
 {
 	switch(stackType) {
 		case MASTER_OUT:
@@ -203,7 +256,7 @@ vector <Plugin *> *PluginHost::getStack(int stackType, Channel *ch)
 		case CHANNEL:
 			return &ch->plugins;
 		default:
-			return NULL;
+			return nullptr;
 	}
 }
 
@@ -211,7 +264,7 @@ vector <Plugin *> *PluginHost::getStack(int stackType, Channel *ch)
 /* -------------------------------------------------------------------------- */
 
 
-unsigned PluginHost::countPlugins(int stackType, Channel *ch)
+unsigned countPlugins(int stackType, Channel *ch)
 {
 	vector <Plugin *> *pStack = getStack(stackType, ch);
 	return pStack->size();
@@ -221,7 +274,7 @@ unsigned PluginHost::countPlugins(int stackType, Channel *ch)
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::countAvailablePlugins()
+int countAvailablePlugins()
 {
   return knownPluginList.getNumTypes();
 }
@@ -230,7 +283,7 @@ int PluginHost::countAvailablePlugins()
 /* -------------------------------------------------------------------------- */
 
 
-unsigned PluginHost::countUnknownPlugins()
+unsigned countUnknownPlugins()
 {
   return unknownPluginList.size();
 }
@@ -239,7 +292,7 @@ unsigned PluginHost::countUnknownPlugins()
 /* -------------------------------------------------------------------------- */
 
 
-PluginHost::PluginInfo PluginHost::getAvailablePluginInfo(int i)
+pluginHost::PluginInfo getAvailablePluginInfo(int i)
 {
   juce::PluginDescription *pd = knownPluginList.getType(i);
   PluginInfo pi;
@@ -251,8 +304,8 @@ PluginHost::PluginInfo PluginHost::getAvailablePluginInfo(int i)
   pi.isInstrument = pd->isInstrument;
 /*
   if (!p) {
-    gu_log("[PluginHost::getAvailablePlugin] unable to create plugin instance!\n");
-    return NULL;
+    gu_log("[pluginHost::getAvailablePlugin] unable to create plugin instance!\n");
+    return nullptr;
   }
   */
   return pi;
@@ -262,7 +315,16 @@ PluginHost::PluginInfo PluginHost::getAvailablePluginInfo(int i)
 /* -------------------------------------------------------------------------- */
 
 
-string PluginHost::getUnknownPluginInfo(int i)
+bool hasMissingPlugins()
+{
+  return missingPlugins;
+};
+
+
+/* -------------------------------------------------------------------------- */
+
+
+string getUnknownPluginInfo(int i)
 {
   return unknownPluginList.at(i);
 }
@@ -271,7 +333,7 @@ string PluginHost::getUnknownPluginInfo(int i)
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::freeStack(int stackType, pthread_mutex_t *mutex, Channel *ch)
+void freeStack(int stackType, pthread_mutex_t *mutex, Channel *ch)
 {
 	vector <Plugin *> *pStack;
 	pStack = getStack(stackType, ch);
@@ -288,20 +350,20 @@ void PluginHost::freeStack(int stackType, pthread_mutex_t *mutex, Channel *ch)
 		pthread_mutex_unlock(mutex);
 		break;
 	}
-  gu_log("[PluginHost::freeStack] stack type=%d freed\n", stackType);
+  gu_log("[pluginHost::freeStack] stack type=%d freed\n", stackType);
 }
 
 
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::processStack(float *buffer, int stackType, Channel *ch)
+void processStack(float *buffer, int stackType, Channel *ch)
 {
 	vector <Plugin *> *pStack = getStack(stackType, ch);
 
 	/* empty stack, stack not found or mixer not ready: do nothing */
 
-	if (pStack == NULL || pStack->size() == 0)
+	if (pStack == nullptr || pStack->size() == 0)
 		return;
 
 	/* converting buffer from Giada to Juce */
@@ -349,13 +411,13 @@ void PluginHost::processStack(float *buffer, int stackType, Channel *ch)
 /* -------------------------------------------------------------------------- */
 
 
-Plugin *PluginHost::getPluginByIndex(int index, int stackType, Channel *ch)
+Plugin *getPluginByIndex(int index, int stackType, Channel *ch)
 {
 	vector <Plugin *> *pStack = getStack(stackType, ch);
 	if (pStack->size() == 0)
-		return NULL;
+		return nullptr;
 	if ((unsigned) index >= pStack->size())
-		return NULL;
+		return nullptr;
 	return pStack->at(index);
 }
 
@@ -363,7 +425,7 @@ Plugin *PluginHost::getPluginByIndex(int index, int stackType, Channel *ch)
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::getPluginIndex(int id, int stackType, Channel *ch)
+int getPluginIndex(int id, int stackType, Channel *ch)
 {
 	vector <Plugin *> *pStack = getStack(stackType, ch);
 	for (unsigned i=0; i<pStack->size(); i++)
@@ -376,7 +438,7 @@ int PluginHost::getPluginIndex(int id, int stackType, Channel *ch)
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::swapPlugin(unsigned indexA, unsigned indexB, int stackType,
+void swapPlugin(unsigned indexA, unsigned indexB, int stackType,
   pthread_mutex_t *mutex, Channel *ch)
 {
 	vector <Plugin *> *pStack = getStack(stackType, ch);
@@ -394,7 +456,7 @@ void PluginHost::swapPlugin(unsigned indexA, unsigned indexB, int stackType,
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::freePlugin(int id, int stackType, pthread_mutex_t *mutex,
+int freePlugin(int id, int stackType, pthread_mutex_t *mutex,
   Channel *ch)
 {
 	vector <Plugin *> *pStack = getStack(stackType, ch);
@@ -420,22 +482,22 @@ int PluginHost::freePlugin(int id, int stackType, pthread_mutex_t *mutex,
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::runDispatchLoop()
+void runDispatchLoop()
 {
   messageManager->runDispatchLoopUntil(10);
-  //gu_log("[PluginHost::runDispatchLoop] %d, hasStopMessageBeenSent=%d\n", r, messageManager->hasStopMessageBeenSent());
+  //gu_log("[pluginHost::runDispatchLoop] %d, hasStopMessageBeenSent=%d\n", r, messageManager->hasStopMessageBeenSent());
 }
 
 
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::freeAllStacks(vector <Channel*> *channels, pthread_mutex_t *mutex)
+void freeAllStacks(vector <Channel*> *channels, pthread_mutex_t *mutex)
 {
-	freeStack(PluginHost::MASTER_OUT, mutex);
-	freeStack(PluginHost::MASTER_IN, mutex);
+	freeStack(pluginHost::MASTER_OUT, mutex);
+	freeStack(pluginHost::MASTER_IN, mutex);
 	for (unsigned i=0; i<channels->size(); i++)
-		freeStack(PluginHost::CHANNEL, mutex, channels->at(i));
+		freeStack(pluginHost::CHANNEL, mutex, channels->at(i));
   missingPlugins = false;
   unknownPluginList.clear();
 }
@@ -444,12 +506,12 @@ void PluginHost::freeAllStacks(vector <Channel*> *channels, pthread_mutex_t *mut
 /* -------------------------------------------------------------------------- */
 
 
-int PluginHost::clonePlugin(Plugin *src, int stackType, pthread_mutex_t *mutex,
+int clonePlugin(Plugin *src, int stackType, pthread_mutex_t *mutex,
   Channel *ch)
 {
 	Plugin *p = addPlugin(src->getUniqueId(), stackType, mutex, ch);
 	if (!p) {
-		gu_log("[PluginHost::clonePlugin] unable to add new plugin to stack!\n");
+		gu_log("[pluginHost::clonePlugin] unable to add new plugin to stack!\n");
 		return 0;
 	}
 
@@ -463,7 +525,7 @@ int PluginHost::clonePlugin(Plugin *src, int stackType, pthread_mutex_t *mutex,
 /* -------------------------------------------------------------------------- */
 
 
-bool PluginHost::doesPluginExist(const string &fid)
+bool doesPluginExist(const string &fid)
 {
   return pluginFormat.doesPluginStillExist(*knownPluginList.getTypeForFile(fid));
 }
@@ -472,7 +534,7 @@ bool PluginHost::doesPluginExist(const string &fid)
 /* -------------------------------------------------------------------------- */
 
 
-void PluginHost::sortPlugins(int method)
+void sortPlugins(int method)
 {
   switch (method) {
     case sortMethod::NAME:
@@ -490,5 +552,7 @@ void PluginHost::sortPlugins(int method)
   }
 }
 
+}}}; // giada::m::pluginHost::
+
 
 #endif // #ifdef WITH_VST
