@@ -2,11 +2,9 @@
  *
  * Giada - Your Hardcore Loopmachine
  *
- * recorder
- *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -27,27 +25,78 @@
  * -------------------------------------------------------------------------- */
 
 
-#include <math.h>
+#include <cassert>
+#include <cmath>
 #include "../utils/log.h"
-#include "mixer.h"
-#include "patch_DEPR_.h"
+#include "const.h"
 #include "sampleChannel.h"
 #include "recorder.h"
 
 
-Recorder::Recorder()
-	: active       (false),
-	  sortedActions(false)
+using std::vector;
+
+
+namespace giada {
+namespace m {
+namespace recorder
 {
-}
+namespace
+{
+/* composite
+A group of two actions (keypress+keyrel, muteon+muteoff) used during the overdub
+process. */
+
+struct composite
+{
+	action a1;
+	action a2;
+} cmp;
+
 
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::init()
+/* fixOverdubTruncation
+Fixes underlying action truncation when overdubbing over a longer action. I.e.:
+  Original:    |#############|
+  Overdub:     ---|#######|---
+  fix:         |#||#######|--- */
+
+void fixOverdubTruncation(const composite &comp, pthread_mutex_t *mixerMutex)
+{
+  action *next = nullptr;
+  int res = getNextAction(comp.a2.chan, comp.a1.type | comp.a2.type, comp.a2.frame,
+    &next);
+  if (res != 1 || next->type != comp.a2.type)
+    return;
+  gu_log("[recorder::fixOverdubTruncation] add truncation at frame %d, type=%d\n",
+    next->frame, next->type);
+  deleteAction(next->chan, next->frame, next->type, false, mixerMutex);
+}
+
+}; // {anonymous}
+
+
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+/* -------------------------------------------------------------------------- */
+
+
+vector<int> frames;
+vector<vector<action*>> global;
+vector<action*> actions;     // used internally
+
+bool active = false;
+bool sortedActions = false;
+
+
+/* -------------------------------------------------------------------------- */
+
+
+void init()
 {
-	sortedActions = false;
 	active = false;
+  sortedActions = false;
 	clearAll();
 }
 
@@ -55,7 +104,7 @@ void Recorder::init()
 /* -------------------------------------------------------------------------- */
 
 
-bool Recorder::canRec(Channel *ch, Mixer *mixer)
+bool canRec(Channel *ch, bool clockRunning, bool mixerRecording)
 {
 	/* NO recording if:
 	 * recorder is inactive
@@ -63,10 +112,10 @@ bool Recorder::canRec(Channel *ch, Mixer *mixer)
 	 * mixer is recording a take somewhere
 	 * channel is empty */
 
-	if (!active           ||
-		  !mixer->running   ||
-			 mixer->recording ||
-			(ch->type == CHANNEL_SAMPLE && ((SampleChannel*)ch)->wave == NULL)
+	if (!active         ||
+		  !clockRunning   ||
+			 mixerRecording ||
+			(ch->type == CHANNEL_SAMPLE && ((SampleChannel*)ch)->wave == nullptr)
 		)
 		return false;
 	return true;
@@ -76,7 +125,7 @@ bool Recorder::canRec(Channel *ch, Mixer *mixer)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::rec(int index, int type, int frame, uint32_t iValue, float fValue)
+void rec(int index, int type, int frame, uint32_t iValue, float fValue)
 {
 	/* make sure frame is even */
 
@@ -133,7 +182,7 @@ void Recorder::rec(int index, int type, int frame, uint32_t iValue, float fValue
 
 	sortedActions = false;
 
-	gu_log("[REC] action recorded, type=%d frame=%d chan=%d iValue=%d (0x%X) fValue=%f\n",
+	gu_log("[recorder::rec] action recorded, type=%d frame=%d chan=%d iValue=%d (0x%X) fValue=%f\n",
 		a->type, a->frame, a->chan, a->iValue, a->iValue, a->fValue);
 	//print();
 }
@@ -142,9 +191,9 @@ void Recorder::rec(int index, int type, int frame, uint32_t iValue, float fValue
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::clearChan(int index)
+void clearChan(int index)
 {
-	gu_log("[REC] clearing chan %d...\n", index);
+	gu_log("[recorder::clearChan] clearing chan %d...\n", index);
 
 	for (unsigned i=0; i<global.size(); i++) {	// for each frame i
 		unsigned j=0;
@@ -167,9 +216,9 @@ void Recorder::clearChan(int index)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::clearAction(int index, char act)
+void clearAction(int index, char act)
 {
-	gu_log("[REC] clearing action %d from chan %d...\n", act, index);
+	gu_log("[recorder::clearAction] clearing action %d from chan %d...\n", act, index);
 	for (unsigned i=0; i<global.size(); i++) {						// for each frame i
 		unsigned j=0;
 		while (true) {                                   // for each action j of frame i
@@ -192,7 +241,7 @@ void Recorder::clearAction(int index, char act)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::deleteAction(int chan, int frame, char type, bool checkValues,
+void deleteAction(int chan, int frame, char type, bool checkValues,
 	pthread_mutex_t *mixerMutex, uint32_t iValue, float fValue)
 {
 	/* make sure frame is even */
@@ -231,17 +280,17 @@ void Recorder::deleteAction(int chan, int frame, char type, bool checkValues,
           break;
         }
         else
-          gu_log("[REC] delete action: waiting for mutex...\n");
+          gu_log("[recorder::deleteAction] waiting for mutex...\n");
       }
 		}
 	}
 	if (found) {
 		optimize();
-		gu_log("[REC] action deleted, type=%d frame=%d chan=%d iValue=%d (%X) fValue=%f\n",
+		gu_log("[recorder::deleteAction] action deleted, type=%d frame=%d chan=%d iValue=%d (%X) fValue=%f\n",
 			type, frame, chan, iValue, iValue, fValue);
 	}
 	else
-		gu_log("[REC] unable to delete action, not found! type=%d frame=%d chan=%d iValue=%d (%X) fValue=%f\n",
+		gu_log("[recorder::deleteAction] unable to delete action, not found! type=%d frame=%d chan=%d iValue=%d (%X) fValue=%f\n",
 			type, frame, chan, iValue, iValue, fValue);
 }
 
@@ -249,7 +298,7 @@ void Recorder::deleteAction(int chan, int frame, char type, bool checkValues,
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::deleteActions(int chan, int frame_a, int frame_b, char type,
+void deleteActions(int chan, int frame_a, int frame_b, char type,
   pthread_mutex_t *mixerMutex)
 {
 	sortActions();
@@ -267,7 +316,7 @@ void Recorder::deleteActions(int chan, int frame_a, int frame_b, char type,
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::clearAll()
+void clearAll()
 {
 	while (global.size() > 0) {
 		for (unsigned i=0; i<global.size(); i++) {
@@ -285,7 +334,7 @@ void Recorder::clearAll()
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::optimize()
+void optimize()
 {
 	/* do something until the i frame is empty. */
 
@@ -307,7 +356,7 @@ void Recorder::optimize()
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::sortActions()
+void sortActions()
 {
 	if (sortedActions)
 		return;
@@ -325,7 +374,7 @@ void Recorder::sortActions()
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::updateBpm(float oldval, float newval, int oldquanto)
+void updateBpm(float oldval, float newval, int oldquanto)
 {
 	for (unsigned i=0; i<frames.size(); i++) {
 
@@ -336,7 +385,7 @@ void Recorder::updateBpm(float oldval, float newval, int oldquanto)
 		 * and the quantizer set to 44100. That would mean two recs completely
 		 * useless. So we compute a reject value ('scarto'): if it's lower
 		 * than 6 frames the new frame is collapsed with a quantized frame. */
-		/** CHECKME - maybe 6 frames are too low */
+		/** XXX - maybe 6 frames are too low */
 
 		if (frames.at(i) != 0) {
 			int scarto = oldquanto % frames.at(i);
@@ -366,7 +415,7 @@ void Recorder::updateBpm(float oldval, float newval, int oldquanto)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::updateSamplerate(int systemRate, int patchRate)
+void updateSamplerate(int systemRate, int patchRate)
 {
 	/* diff ratio: systemRate / patchRate
 	 * e.g.  44100 / 96000 = 0.4... */
@@ -374,12 +423,12 @@ void Recorder::updateSamplerate(int systemRate, int patchRate)
 	if (systemRate == patchRate)
 		return;
 
-	gu_log("[REC] systemRate (%d) != patchRate (%d), converting...\n", systemRate, patchRate);
+	gu_log("[recorder::updateSamplerate] systemRate (%d) != patchRate (%d), converting...\n", systemRate, patchRate);
 
 	float ratio = systemRate / (float) patchRate;
 	for (unsigned i=0; i<frames.size(); i++) {
 
-		gu_log("[REC]    oldFrame = %d", frames.at(i));
+		gu_log("[recorder::updateSamplerate]    oldFrame = %d", frames.at(i));
 
 		float newFrame = frames.at(i);
 		newFrame = floorf(newFrame * ratio);
@@ -406,7 +455,7 @@ void Recorder::updateSamplerate(int systemRate, int patchRate)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::expand(int old_fpb, int new_fpb)
+void expand(int old_fpb, int new_fpb)
 {
 	/* this algorithm requires multiple passages if we expand from e.g. 2
 	 * to 16 beats, precisely 16 / 2 - 1 = 7 times (-1 is the first group,
@@ -429,7 +478,7 @@ void Recorder::expand(int old_fpb, int new_fpb)
 			}
 		}
 	}
-	gu_log("[REC] expanded recs\n");
+	gu_log("[recorder::expand] expanded recs\n");
 	//print();
 }
 
@@ -437,7 +486,7 @@ void Recorder::expand(int old_fpb, int new_fpb)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::shrink(int new_fpb)
+void shrink(int new_fpb)
 {
 	/* easier than expand(): here we delete eveything beyond old_framesPerBars. */
 
@@ -456,7 +505,7 @@ void Recorder::shrink(int new_fpb)
 			i++;
 	}
 	optimize();
-	gu_log("[REC] shrinked recs\n");
+	gu_log("[recorder::shrink] shrinked recs\n");
 	//print();
 }
 
@@ -464,7 +513,7 @@ void Recorder::shrink(int new_fpb)
 /* -------------------------------------------------------------------------- */
 
 
-bool Recorder::hasActions(int chanIndex)
+bool hasActions(int chanIndex)
 {
   if (global.size() == 0)
     return false;
@@ -481,7 +530,7 @@ bool Recorder::hasActions(int chanIndex)
 /* -------------------------------------------------------------------------- */
 
 
-int Recorder::getNextAction(int chan, char type, int frame, action **out,
+int getNextAction(int chan, char type, int frame, action **out,
 	uint32_t iValue)
 {
 	sortActions();  // mandatory
@@ -511,7 +560,7 @@ int Recorder::getNextAction(int chan, char type, int frame, action **out,
 /* -------------------------------------------------------------------------- */
 
 
-int Recorder::getAction(int chan, char action, int frame, struct action **out)
+int getAction(int chan, char action, int frame, struct action **out)
 {
 	for (unsigned i=0; i<global.size(); i++)
 		for (unsigned j=0; j<global.at(i).size(); j++)
@@ -529,18 +578,17 @@ int Recorder::getAction(int chan, char action, int frame, struct action **out)
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::startOverdub(int index, char actionMask, int frame,
-  unsigned bufferSize)
+void startOverdub(int index, char actionMask, int frame, unsigned bufferSize)
 {
 	/* prepare the composite struct */
 
-	if (actionMask == ACTION_KEYS) {
-		cmp.a1.type = ACTION_KEYPRESS;
-		cmp.a2.type = ACTION_KEYREL;
+	if (actionMask == G_ACTION_KEYS) {
+		cmp.a1.type = G_ACTION_KEYPRESS;
+		cmp.a2.type = G_ACTION_KEYREL;
 	}
 	else {
-		cmp.a1.type = ACTION_MUTEON;
-		cmp.a2.type = ACTION_MUTEOFF;
+		cmp.a1.type = G_ACTION_MUTEON;
+		cmp.a2.type = G_ACTION_MUTEOFF;
 	}
 	cmp.a1.chan  = index;
 	cmp.a2.chan  = index;
@@ -552,14 +600,14 @@ void Recorder::startOverdub(int index, char actionMask, int frame,
 
 	rec(index, cmp.a1.type, frame);
 
-	action *act = NULL;
+	action *act = nullptr;
 	int res = getNextAction(index, cmp.a1.type | cmp.a2.type, cmp.a1.frame, &act);
 	if (res == 1) {
 		if (act->type == cmp.a2.type) {
 			int truncFrame = cmp.a1.frame - bufferSize;
 			if (truncFrame < 0)
 				truncFrame = 0;
-			gu_log("[REC] add truncation at frame %d, type=%d\n", truncFrame, cmp.a2.type);
+			gu_log("[recorder::startOverdub] add truncation at frame %d, type=%d\n", truncFrame, cmp.a2.type);
 			rec(index, cmp.a2.type, truncFrame);
 		}
 	}
@@ -569,61 +617,52 @@ void Recorder::startOverdub(int index, char actionMask, int frame,
 /* -------------------------------------------------------------------------- */
 
 
-void Recorder::stopOverdub(Mixer *mixer)
+void stopOverdub(int currentFrame, int totalFrames, pthread_mutex_t *mixerMutex)
 {
-	cmp.a2.frame  = mixer->currentFrame;
+	cmp.a2.frame  = currentFrame;
 	bool ringLoop = false;
 	bool nullLoop = false;
 
-	/* ring loop verification, i.e. a composite action with key_press at
-	 * frame N and key_release at frame M, with M <= N */
+	/* Check for ring loops or null loops. Ring loop: a composite action with
+  key_press at frame N and key_release at frame M, with M <= N.
+  Null loop: a composite action that begins and ends on the very same frame,
+  i.e. with 0 size. Very unlikely.
+  If ring loop: record the last action at the end of the sequencer (that
+  is 'totalFrames').
+  If null loop: remove previous action and do nothing. Also make sure to avoid
+  underlying action truncation, if the null loop occurs inside a composite
+  action. */
 
-	if (cmp.a2.frame < cmp.a1.frame) {
+	if (cmp.a2.frame < cmp.a1.frame) {  // ring loop
 		ringLoop = true;
-		gu_log("[REC] ring loop! frame1=%d < frame2=%d\n", cmp.a1.frame, cmp.a2.frame);
-		rec(cmp.a2.chan, cmp.a2.type, mixer->totalFrames); 	// record at the end of the sequencer
+		gu_log("[recorder::stopOverdub] ring loop! frame1=%d < frame2=%d\n", cmp.a1.frame, cmp.a2.frame);
+		rec(cmp.a2.chan, cmp.a2.type, totalFrames);
 	}
 	else
-	if (cmp.a2.frame == cmp.a1.frame) {
+	if (cmp.a2.frame == cmp.a1.frame) { // null loop
 		nullLoop = true;
-		gu_log("[REC]  null loop! frame1=%d == frame2=%d\n", cmp.a1.frame, cmp.a2.frame);
-		deleteAction(cmp.a1.chan, cmp.a1.frame, cmp.a1.type, false, &mixer->mutex_recs); // false == don't check values
-	}
+		gu_log("[recorder::stopOverdub] null loop! frame1=%d == frame2=%d\n", cmp.a1.frame, cmp.a2.frame);
+		deleteAction(cmp.a1.chan, cmp.a1.frame, cmp.a1.type, false, mixerMutex); // false == don't check values
+    fixOverdubTruncation(cmp, mixerMutex);
+  }
 
-	/* remove any nested action between keypress----keyrel, then record */
+  if (nullLoop)
+    return;
 
-	if (!nullLoop) {
-		deleteActions(cmp.a2.chan, cmp.a1.frame, cmp.a2.frame, cmp.a1.type, &mixer->mutex_recs);
-		deleteActions(cmp.a2.chan, cmp.a1.frame, cmp.a2.frame, cmp.a2.type, &mixer->mutex_recs);
-	}
+	/* Remove any nested action between keypress----keyrel. */
 
-	if (!ringLoop && !nullLoop) {
-		rec(cmp.a2.chan, cmp.a2.type, cmp.a2.frame);
+	deleteActions(cmp.a2.chan, cmp.a1.frame, cmp.a2.frame, cmp.a1.type, mixerMutex);
+	deleteActions(cmp.a2.chan, cmp.a1.frame, cmp.a2.frame, cmp.a2.type, mixerMutex);
 
-		/* avoid underlying action truncation, if keyrel happens inside a
-		* composite action */
+  if (ringLoop)
+    return;
 
-		action *act = NULL;
-		int res = getNextAction(cmp.a2.chan, cmp.a1.type | cmp.a2.type, cmp.a2.frame, &act);
-		if (res == 1 && act->type == cmp.a2.type) {
-			gu_log("[REC] add truncation at frame %d, type=%d\n", act->frame, act->type);
-			deleteAction(act->chan, act->frame, act->type, false, &mixer->mutex_recs); // false == don't check values
-		}
-	}
+  /* Record second part of the composite action. Also make sure to avoid
+  underlying action truncation, if keyrel happens inside a composite action. */
+
+	rec(cmp.a2.chan, cmp.a2.type, cmp.a2.frame);
+  fixOverdubTruncation(cmp, mixerMutex);
 }
 
 
-/* -------------------------------------------------------------------------- */
-
-
-void Recorder::print()
-{
-	gu_log("[REC] ** print debug **\n");
-	for (unsigned i=0; i<global.size(); i++) {
-		gu_log("  frame %d\n", frames.at(i));
-		for (unsigned j=0; j<global.at(i).size(); j++) {
-			gu_log("    action %d | chan %d | frame %d\n", global.at(i).at(j)->type,
-        global.at(i).at(j)->chan, global.at(i).at(j)->frame);
-		}
-	}
-}
+}}}; // giada::m::recorder::
