@@ -2,11 +2,9 @@
  *
  * Giada - Your Hardcore Loopmachine
  *
- * gd_mainWindow
- *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -31,7 +29,6 @@
 #include "../../core/const.h"
 #include "../../core/init.h"
 #include "../../utils/gui.h"
-#include "../elems/ge_mixed.h"
 #include "../elems/basics/boxtypes.h"
 #include "../elems/mainWindow/mainIO.h"
 #include "../elems/mainWindow/mainMenu.h"
@@ -47,7 +44,7 @@ extern gdMainWindow *G_MainWin;
 
 
 gdMainWindow::gdMainWindow(int W, int H, const char *title, int argc, char **argv)
-	: gWindow(W, H, title)
+	: gdWindow(W, H, title)
 {
 	Fl::visible_focus(0);
 
