@@ -2,11 +2,9 @@
  *
  * Giada - Your Hardcore Loopmachine
  *
- * gd_pluginList
- *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -42,32 +40,31 @@
 #include "../../glue/plugin.h"
 #include "../../utils/log.h"
 #include "../../utils/string.h"
-#include "../elems/ge_mixed.h"
 #include "../elems/basics/boxtypes.h"
+#include "../elems/basics/idButton.h"
+#include "../elems/basics/statusButton.h"
+#include "../elems/basics/choice.h"
 #include "../elems/mainWindow/mainIO.h"
 #include "../elems/mainWindow/keyboard/channel.h"
 #include "gd_pluginList.h"
 #include "gd_pluginChooser.h"
 #include "gd_pluginWindow.h"
 #include "gd_pluginWindowGUI.h"
-#include "gd_browser.h"
 #include "gd_mainWindow.h"
 
 
-extern Conf          G_Conf;
-extern Mixer         G_Mixer;
-extern PluginHost    G_PluginHost;
 extern gdMainWindow *G_MainWin;
 
 
 using std::string;
+using namespace giada::m;
 
 
 gdPluginList::gdPluginList(int stackType, Channel *ch)
-  : gWindow(468, 204), ch(ch), stackType(stackType)
+  : gdWindow(468, 204), ch(ch), stackType(stackType)
 {
-	if (G_Conf.pluginListX)
-		resize(G_Conf.pluginListX, G_Conf.pluginListY, w(), h());
+	if (conf::pluginListX)
+		resize(conf::pluginListX, conf::pluginListY, w(), h());
 
 	list = new Fl_Scroll(8, 8, 476, 188);
 	list->type(Fl_Scroll::VERTICAL);
@@ -86,10 +83,10 @@ gdPluginList::gdPluginList(int stackType, Channel *ch)
   /* TODO - awful stuff... we should subclass into gdPluginListChannel and
   gdPluginListMaster */
 
-	if (stackType == PluginHost::MASTER_OUT)
+	if (stackType == pluginHost::MASTER_OUT)
 		label("Master Out Plugins");
 	else
-	if (stackType == PluginHost::MASTER_IN)
+	if (stackType == pluginHost::MASTER_IN)
 		label("Master In Plugins");
 	else {
     string l = "Channel " + gu_itoa(ch->index+1) + " Plugins";
@@ -106,8 +103,8 @@ gdPluginList::gdPluginList(int stackType, Channel *ch)
 
 gdPluginList::~gdPluginList()
 {
-	G_Conf.pluginListX = x();
-	G_Conf.pluginListY = y();
+	conf::pluginListX = x();
+	conf::pluginListY = y();
 }
 
 
@@ -126,8 +123,8 @@ void gdPluginList::cb_refreshList(Fl_Widget *v, void *p)
 	 * by calling the parent (pluginList) and telling it to delete its
 	 * subwindow (i.e. gdBrowser). */
 
-	gWindow *child = (gWindow*) v;
-	if (child->getParent() != NULL)
+	gdWindow *child = (gdWindow*) v;
+	if (child->getParent() != nullptr)
 		(child->getParent())->delSubWindow(child);
 
 	/* finally refresh plugin list: void *p is a pointer to gdPluginList.
@@ -144,13 +141,13 @@ void gdPluginList::cb_refreshList(Fl_Widget *v, void *p)
 
 void gdPluginList::__cb_addPlugin()
 {
-	/* the usual callback that gWindow adds to each subwindow in this case
+	/* the usual callback that gdWindow adds to each subwindow in this case
 	 * is not enough, because when we close the browser the plugin list
 	 * must be redrawn. We have a special callback, cb_refreshList, which
 	 * we add to gdPluginChooser. It does exactly what we need. */
 
-  gdPluginChooser *pc = new gdPluginChooser(G_Conf.pluginChooserX,
-      G_Conf.pluginChooserY, G_Conf.pluginChooserW, G_Conf.pluginChooserH,
+  gdPluginChooser *pc = new gdPluginChooser(conf::pluginChooserX,
+      conf::pluginChooserY, conf::pluginChooserW, conf::pluginChooserH,
       stackType, ch);
   addSubWindow(pc);
   pc->callback(cb_refreshList, (void*)this);	// 'this' refers to gdPluginList
@@ -168,21 +165,21 @@ void gdPluginList::refreshList()
 	list->scroll_to(0, 0);
 
 	/* add new buttons, as many as the plugin in pluginHost::stack + 1,
-	 * the 'add new' button. Warning: if ch == NULL we are working with
+	 * the 'add new' button. Warning: if ch == nullptr we are working with
 	 * master in/master out stacks. */
 
-	int numPlugins = G_PluginHost.countPlugins(stackType, ch);
+	int numPlugins = pluginHost::countPlugins(stackType, ch);
 	int i = 0;
 
 	while (i<numPlugins) {
-		Plugin   *pPlugin = G_PluginHost.getPluginByIndex(i, stackType, ch);
+		Plugin   *pPlugin = pluginHost::getPluginByIndex(i, stackType, ch);
 		gdPlugin *gdp     = new gdPlugin(this, pPlugin, list->x(), list->y()-list->yposition()+(i*24), 800);
 		list->add(gdp);
 		i++;
 	}
 
 	int addPlugY = numPlugins == 0 ? 90 : list->y()-list->yposition()+(i*24);
-	addPlugin = new gClick(8, addPlugY, 452, 20, "-- add new plugin --");
+	addPlugin = new geButton(8, addPlugY, 452, 20, "-- add new plugin --");
 	addPlugin->callback(cb_addPlugin, (void*)this);
 	list->add(addPlugin);
 
@@ -201,17 +198,17 @@ void gdPluginList::refreshList()
   /* TODO - awful stuff... we should subclass into gdPluginListChannel and
   gdPluginListMaster */
 
-	if (stackType == PluginHost::MASTER_OUT) {
+	if (stackType == pluginHost::MASTER_OUT) {
     G_MainWin->mainIO->setMasterFxOutFull(
-			G_PluginHost.countPlugins(stackType, ch) > 0);
+			pluginHost::countPlugins(stackType, ch) > 0);
   }
 	else
-	if (stackType == PluginHost::MASTER_IN) {
+	if (stackType == pluginHost::MASTER_IN) {
     G_MainWin->mainIO->setMasterFxInFull(
-			G_PluginHost.countPlugins(stackType, ch) > 0);
+			pluginHost::countPlugins(stackType, ch) > 0);
   }
 	else {
-    ch->guiChannel->fx->full = G_PluginHost.countPlugins(stackType, ch) > 0;
+    ch->guiChannel->fx->status = pluginHost::countPlugins(stackType, ch) > 0;
     ch->guiChannel->fx->redraw();
   }
 }
@@ -226,12 +223,12 @@ gdPlugin::gdPlugin(gdPluginList *gdp, Plugin *p, int X, int Y, int W)
 	: Fl_Group(X, Y, W, 20), pParent(gdp), pPlugin (p)
 {
 	begin();
-	button    = new gButton(8, y(), 220, 20);
-	program   = new gChoice(button->x()+button->w()+4, y(), 132, 20);
-	bypass    = new gButton(program->x()+program->w()+4, y(), 20, 20);
-	shiftUp   = new gButton(bypass->x()+bypass->w()+4, y(), 20, 20, "", fxShiftUpOff_xpm, fxShiftUpOn_xpm);
-	shiftDown = new gButton(shiftUp->x()+shiftUp->w()+4, y(), 20, 20, "", fxShiftDownOff_xpm, fxShiftDownOn_xpm);
-	remove    = new gButton(shiftDown->x()+shiftDown->w()+4, y(), 20, 20, "", fxRemoveOff_xpm, fxRemoveOn_xpm);
+	button    = new geIdButton(8, y(), 220, 20);
+	program   = new geChoice(button->x()+button->w()+4, y(), 132, 20);
+	bypass    = new geIdButton(program->x()+program->w()+4, y(), 20, 20);
+	shiftUp   = new geIdButton(bypass->x()+bypass->w()+4, y(), 20, 20, "", fxShiftUpOff_xpm, fxShiftUpOn_xpm);
+	shiftDown = new geIdButton(shiftUp->x()+shiftUp->w()+4, y(), 20, 20, "", fxShiftDownOff_xpm, fxShiftDownOn_xpm);
+	remove    = new geIdButton(shiftDown->x()+shiftDown->w()+4, y(), 20, 20, "", fxRemoveOff_xpm, fxRemoveOn_xpm);
 	end();
 
 	button->copy_label(pPlugin->getName().c_str());
@@ -277,10 +274,10 @@ void gdPlugin::__cb_shiftUp()
 {
 	/*nothing to do if there's only one plugin */
 
-	if (G_PluginHost.countPlugins(pParent->stackType, pParent->ch) == 1)
+	if (pluginHost::countPlugins(pParent->stackType, pParent->ch) == 1)
 		return;
 
-	int pluginIndex = G_PluginHost.getPluginIndex(pPlugin->getId(),
+	int pluginIndex = pluginHost::getPluginIndex(pPlugin->getId(),
     pParent->stackType, pParent->ch);
 
 	if (pluginIndex == 0)  // first of the stack, do nothing
@@ -298,11 +295,11 @@ void gdPlugin::__cb_shiftDown()
 {
 	/*nothing to do if there's only one plugin */
 
-	if (G_PluginHost.countPlugins(pParent->stackType, pParent->ch) == 1)
+	if (pluginHost::countPlugins(pParent->stackType, pParent->ch) == 1)
 		return;
 
-	unsigned pluginIndex = G_PluginHost.getPluginIndex(pPlugin->getId(), pParent->stackType, pParent->ch);
-	unsigned stackSize   = (G_PluginHost.getStack(pParent->stackType, pParent->ch))->size();
+	unsigned pluginIndex = pluginHost::getPluginIndex(pPlugin->getId(), pParent->stackType, pParent->ch);
+	unsigned stackSize   = (pluginHost::getStack(pParent->stackType, pParent->ch))->size();
 
 	if (pluginIndex == stackSize-1)  // last one in the stack, do nothing
 		return;
@@ -333,7 +330,7 @@ void gdPlugin::__cb_openPluginWindow()
   /* the new pluginWindow has id = id_plugin + 1, because id=0 is reserved
   * for the parent window 'add plugin'. */
 
-  gWindow *w;
+  gdWindow *w;
   if (pPlugin->hasEditor()) {
     if (pPlugin->isEditorOpen()) {
       gu_log("[gdPlugin::__cb_openPluginWindow] plugin has editor but it's already visible\n");
