@@ -7,7 +7,7 @@
  *
  * ---------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -31,23 +31,26 @@
 #ifdef WITH_VST
 
 
-#ifndef __GD_PLUGIN_WINDOW_GUI_H__
-#define __GD_PLUGIN_WINDOW_GUI_H__
+#ifndef GD_PLUGIN_WINDOW_GUI_H
+#define GD_PLUGIN_WINDOW_GUI_H
 
 
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
-#include "../elems/ge_window.h"
+#include "window.h"
 #if defined(__APPLE__)
 	#include <Carbon/Carbon.h>
 #endif
 
 
-class gdPluginWindowGUI : public gWindow
+class Plugin;
+
+
+class gdPluginWindowGUI : public gdWindow
 {
 private:
 
-	class Plugin *pPlugin;
+	Plugin *pPlugin;
 
 	static void cb_close    (Fl_Widget *v, void *p);
 	static void cb_refresh  (void *data);
@@ -61,31 +64,6 @@ public:
 };
 
 
-/* -------------------------------------------------------------------------- */
-
-#if 0
-#if defined(__APPLE__)
-
-class gdPluginWindowGUImac : public gWindow
-{
-private:
-
-	static pascal OSStatus windowHandler(EventHandlerCallRef ehc, EventRef e, void *data);
-	inline pascal OSStatus __wh(EventHandlerCallRef ehc, EventRef e);
-
-	class Plugin *pPlugin;
-	WindowRef carbonWindow;
-	bool open;
-
-public:
-
-	gdPluginWindowGUImac(Plugin *pPlugin);
-	~gdPluginWindowGUImac();
-};
-
-#endif
-#endif
-
 #endif // include guard
 
 #endif // #ifdef WITH_VST
