@@ -1,12 +1,10 @@
-/* ---------------------------------------------------------------------
+/* -----------------------------------------------------------------------------
  *
  * Giada - Your Hardcore Loopmachine
  *
- * gd_warnings
+ * -----------------------------------------------------------------------------
  *
- * ---------------------------------------------------------------------
- *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -24,22 +22,29 @@
  * along with Giada - Your Hardcore Loopmachine. If not, see
  * <http://www.gnu.org/licenses/>.
  *
- * ------------------------------------------------------------------ */
+ * -------------------------------------------------------------------------- */
 
 
+#include <FL/Fl.H>
+#include <FL/Fl_Window.H>
+#include "../../utils/gui.h"
 #include "../../core/const.h"
+#include "../elems/basics/button.h"
+#include "../elems/basics/box.h"
+#include "window.h"
 #include "gd_warnings.h"
 
 
-void gdAlert(const char *c) {
+void gdAlert(const char *c)
+{
 	Fl_Window *modal = new Fl_Window(
 			(Fl::w() / 2) - 150,
 			(Fl::h() / 2) - 47,
 			300, 90, "Alert");
 	modal->set_modal();
 	modal->begin();
-		gBox *box = new gBox(10, 10, 280, 40, c);
-		gClick *b = new gClick(210, 60, 80, 20, "Close");
+		geBox *box = new geBox(10, 10, 280, 40, c);
+		geButton *b = new geButton(210, 60, 80, 20, "Close");
 	modal->end();
 	box->labelsize(GUI_FONT_SIZE_BASE);
 	b->callback(__cb_window_closer, (void *)modal);
@@ -49,16 +54,17 @@ void gdAlert(const char *c) {
 }
 
 
-int gdConfirmWin(const char *title, const char *msg) {
+int gdConfirmWin(const char *title, const char *msg)
+{
 	Fl_Window *win = new Fl_Window(
 			(Fl::w() / 2) - 150,
 			(Fl::h() / 2) - 47,
 			300, 90, title);
 	win->set_modal();
 	win->begin();
-		new gBox(10, 10, 280, 40, msg);
-		gClick *ok = new gClick(212, 62, 80, 20, "Ok");
-		gClick *ko = new gClick(124, 62, 80, 20, "Cancel");
+		new geBox(10, 10, 280, 40, msg);
+		geButton *ok = new geButton(212, 62, 80, 20, "Ok");
+		geButton *ko = new geButton(124, 62, 80, 20, "Cancel");
 	win->end();
 	ok->shortcut(FL_Enter);
 	gu_setFavicon(win);
