@@ -7,7 +7,7 @@
  *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -28,8 +28,8 @@
  * -------------------------------------------------------------------------- */
 
 
-#ifndef __GE_SCROLL_H__
-#define __GE_SCROLL_H__
+#ifndef GE_SCROLL_H
+#define GE_SCROLL_H
 
 
 #include <FL/Fl_Scroll.H>
