@@ -2,11 +2,9 @@
  *
  * Giada - Your Hardcore Loopmachine
  *
- * ge_sampleChannel
- *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -29,54 +27,171 @@
 
 #include "../../../../core/mixer.h"
 #include "../../../../core/conf.h"
-#include "../../../../core/recorder.h"
+#include "../../../../core/clock.h"
 #include "../../../../core/graphics.h"
 #include "../../../../core/wave.h"
 #include "../../../../core/sampleChannel.h"
-#include "../../../../glue/main.h"
 #include "../../../../glue/io.h"
 #include "../../../../glue/channel.h"
+#include "../../../../glue/recorder.h"
 #include "../../../../glue/storage.h"
-#include "../../../../utils/string.h"
+#include "../../../../utils/gui.h"
 #include "../../../dialogs/gd_mainWindow.h"
 #include "../../../dialogs/gd_keyGrabber.h"
-#include "../../../dialogs/gd_editor.h"
+#include "../../../dialogs/sampleEditor.h"
 #include "../../../dialogs/gd_actionEditor.h"
 #include "../../../dialogs/gd_warnings.h"
-#include "../../../dialogs/gd_browser.h"
+#include "../../../dialogs/browser/browserSave.h"
+#include "../../../dialogs/browser/browserLoad.h"
 #include "../../../dialogs/midiIO/midiOutputSampleCh.h"
 #include "../../../dialogs/midiIO/midiInputChannel.h"
 #include "../../basics/boxtypes.h"
+#include "../../basics/idButton.h"
+#include "../../basics/statusButton.h"
+#include "../../basics/dial.h"
 #include "channelStatus.h"
 #include "channelMode.h"
+#include "sampleChannelButton.h"
 #include "keyboard.h"
 #include "sampleChannel.h"
 
 
-extern Mixer 		     G_Mixer;
-extern Conf  		     G_Conf;
-extern Recorder			 G_Recorder;
 extern gdMainWindow *G_MainWin;
 
 
+using namespace giada::m;
+
+
+namespace
+{
+enum class Menu
+{
+  INPUT_MONITOR = 0,
+  LOAD_SAMPLE,
+  EXPORT_SAMPLE,
+  SETUP_KEYBOARD_INPUT,
+  SETUP_MIDI_INPUT,
+  SETUP_MIDI_OUTPUT,
+  EDIT_SAMPLE,
+  EDIT_ACTIONS,
+  CLEAR_ACTIONS,
+  CLEAR_ACTIONS_ALL,
+  CLEAR_ACTIONS_MUTE,
+  CLEAR_ACTIONS_VOLUME,
+  CLEAR_ACTIONS_START_STOP,
+  __END_SUBMENU__,
+  CLONE_CHANNEL,
+  FREE_CHANNEL,
+  DELETE_CHANNEL
+};
+
+
+/* -------------------------------------------------------------------------- */
+
+
+void menuCallback(Fl_Widget *w, void *v)
+{
+  geSampleChannel *gch = static_cast<geSampleChannel*>(w);
+  Menu selectedItem = (Menu) (intptr_t) v;
+
+  switch (selectedItem)
+  {
+    case Menu::INPUT_MONITOR: {
+      glue_toggleInputMonitor(gch->ch);
+      break;
+    }
+    case Menu::LOAD_SAMPLE: {
+      gdWindow *w = new gdBrowserLoad(conf::browserX, conf::browserY,
+        conf::browserW, conf::browserH, "Browse sample",
+        conf::samplePath.c_str(), glue_loadSample, gch->ch);
+      gu_openSubWindow(G_MainWin, w, WID_FILE_BROWSER);
+      break;
+    }
+    case Menu::EXPORT_SAMPLE: {
+      gdWindow *w = new gdBrowserSave(conf::browserX, conf::browserY,
+        conf::browserW, conf::browserH, "Save sample",
+        conf::samplePath.c_str(), "", glue_saveSample, gch->ch);
+      gu_openSubWindow(G_MainWin, w, WID_FILE_BROWSER);
+  		break;
+    }
+    case Menu::SETUP_KEYBOARD_INPUT: {
+  		new gdKeyGrabber(gch->ch); // FIXME - use gu_openSubWindow
+  		break;
+    }
+    case Menu::SETUP_MIDI_INPUT: {
+      gu_openSubWindow(G_MainWin, new gdMidiInputChannel(gch->ch), 0);
+      break;
+    }
+    case Menu::SETUP_MIDI_OUTPUT: {
+      gu_openSubWindow(G_MainWin, new gdMidiOutputSampleCh(static_cast<SampleChannel*>(gch->ch)), 0);
+      break;
+    }
+    case Menu::EDIT_SAMPLE: {
+      gu_openSubWindow(G_MainWin, new gdSampleEditor(static_cast<SampleChannel*>(gch->ch)), WID_SAMPLE_EDITOR);
+      break;
+    }
+    case Menu::EDIT_ACTIONS: {
+      gu_openSubWindow(G_MainWin, new gdActionEditor(gch->ch), WID_ACTION_EDITOR);
+      break;
+    }
+    case Menu::CLEAR_ACTIONS:
+    case Menu::__END_SUBMENU__:
+      break;
+    case Menu::CLEAR_ACTIONS_ALL: {
+      glue_clearAllActions(gch);
+      break;
+    }
+    case Menu::CLEAR_ACTIONS_MUTE: {
+      glue_clearMuteActions(gch);
+      break;
+    }
+    case Menu::CLEAR_ACTIONS_VOLUME: {
+      glue_clearVolumeActions(gch);
+      break;
+    }
+    case Menu::CLEAR_ACTIONS_START_STOP: {
+      glue_clearStartStopActions(gch);
+      break;
+    }
+    case Menu::CLONE_CHANNEL: {
+      glue_cloneChannel(gch->ch);
+      break;
+    }
+    case Menu::FREE_CHANNEL: {
+  		glue_freeChannel(gch->ch);
+      break;
+    }
+    case Menu::DELETE_CHANNEL: {
+      glue_deleteChannel(gch->ch);
+      break;
+    }
+  }
+}
+
+}; // {namespace}
+
+
+/* -------------------------------------------------------------------------- */
+
+
 geSampleChannel::geSampleChannel(int X, int Y, int W, int H, SampleChannel *ch)
 	: geChannel(X, Y, W, H, CHANNEL_SAMPLE, (Channel*) ch)
 {
 	begin();
 
-	button      = new gButton(x(), y(), 20, 20, "", channelStop_xpm, channelPlay_xpm);
-	arm         = new gClick(button->x()+button->w()+4, y(), 20, 20, "", armOff_xpm, armOn_xpm);
+	button      = new geIdButton(x(), y(), 20, 20, "", channelStop_xpm, channelPlay_xpm);
+	arm         = new geButton(button->x()+button->w()+4, y(), 20, 20, "", armOff_xpm, armOn_xpm);
 	status      = new geChannelStatus(arm->x()+arm->w()+4, y(), 20, 20, ch);
 	mainButton  = new geSampleChannelButton(status->x()+status->w()+4, y(), 20, 20, "-- no sample --");
-	readActions = new gClick(mainButton->x()+mainButton->w()+4, y(), 20, 20, "", readActionOff_xpm, readActionOn_xpm);
+	readActions = new geButton(mainButton->x()+mainButton->w()+4, y(), 20, 20, "", readActionOff_xpm, readActionOn_xpm);
 	modeBox     = new geChannelMode(readActions->x()+readActions->w()+4, y(), 20, 20, ch);
-	mute        = new gClick(modeBox->x()+modeBox->w()+4, y(), 20, 20, "", muteOff_xpm, muteOn_xpm);
-	solo        = new gClick(mute->x()+mute->w()+4, y(), 20, 20, "", soloOff_xpm, soloOn_xpm);
+	mute        = new geButton(modeBox->x()+modeBox->w()+4, y(), 20, 20, "", muteOff_xpm, muteOn_xpm);
+	solo        = new geButton(mute->x()+mute->w()+4, y(), 20, 20, "", soloOff_xpm, soloOn_xpm);
 #ifdef WITH_VST
-	fx          = new gFxButton(solo->x()+solo->w()+4, y(), 20, 20, fxOff_xpm, fxOn_xpm);
-	vol         = new gDial(fx->x()+fx->w()+4, y(), 20, 20);
+	fx          = new geStatusButton(solo->x()+solo->w()+4, y(), 20, 20, fxOff_xpm, fxOn_xpm);
+	vol         = new geDial(fx->x()+fx->w()+4, y(), 20, 20);
 #else
-	vol         = new gDial(solo->x()+solo->w()+4, y(), 20, 20);
+	vol         = new geDial(solo->x()+solo->w()+4, y(), 20, 20);
 #endif
 
 	end();
@@ -141,48 +256,45 @@ void geSampleChannel::__cb_openMenu()
 	/* If you're recording (input or actions) no menu is allowed; you can't do
 	anything, especially deallocate the channel */
 
-	if (G_Mixer.recording || G_Recorder.active)
+	if (mixer::recording || recorder::active)
 		return;
 
-	/* the following is a trash workaround for a FLTK menu. We need a gMenu
-	 * widget asap */
-
 	Fl_Menu_Item rclick_menu[] = {
-		{"Load new sample..."},                     // 0
-		{"Export sample to file..."},               // 1
-		{"Setup keyboard input..."},                // 2
-		{"Setup MIDI input..."},                    // 3
-		{"Setup MIDI output..."},                   // 4
-		{"Edit sample..."},                         // 5
-		{"Edit actions..."},                        // 6
-		{"Clear actions", 0, 0, 0, FL_SUBMENU},     // 7
-			{"All"},                                  // 8
-			{"Mute"},                                 // 9
-			{"Volume"},                               // 10
-			{"Start/Stop"},                           // 11
-			{0},                                      // 12
-		{"Clone channel"},                          // 13
-		{"Free channel"},                           // 14
-		{"Delete channel"},                         // 15
+		{"Input monitor",            0, menuCallback, (void*) Menu::INPUT_MONITOR,
+      FL_MENU_TOGGLE | FL_MENU_DIVIDER | (static_cast<SampleChannel*>(ch)->inputMonitor ? FL_MENU_VALUE : 0)},
+		{"Load new sample...",       0, menuCallback, (void*) Menu::LOAD_SAMPLE},
+		{"Export sample to file...", 0, menuCallback, (void*) Menu::EXPORT_SAMPLE},
+		{"Setup keyboard input...",  0, menuCallback, (void*) Menu::SETUP_KEYBOARD_INPUT},
+		{"Setup MIDI input...",      0, menuCallback, (void*) Menu::SETUP_MIDI_INPUT},
+		{"Setup MIDI output...",     0, menuCallback, (void*) Menu::SETUP_MIDI_OUTPUT},
+		{"Edit sample...",           0, menuCallback, (void*) Menu::EDIT_SAMPLE},
+		{"Edit actions...",          0, menuCallback, (void*) Menu::EDIT_ACTIONS},
+		{"Clear actions",            0, menuCallback, (void*) Menu::CLEAR_ACTIONS, FL_SUBMENU},
+			{"All",        0, menuCallback, (void*) Menu::CLEAR_ACTIONS_ALL},
+			{"Mute",       0, menuCallback, (void*) Menu::CLEAR_ACTIONS_MUTE},
+			{"Volume",     0, menuCallback, (void*) Menu::CLEAR_ACTIONS_VOLUME},
+			{"Start/Stop", 0, menuCallback, (void*) Menu::CLEAR_ACTIONS_START_STOP},
+			{0},
+		{"Clone channel",  0, menuCallback, (void*) Menu::CLONE_CHANNEL},
+		{"Free channel",   0, menuCallback, (void*) Menu::FREE_CHANNEL},
+		{"Delete channel", 0, menuCallback, (void*) Menu::DELETE_CHANNEL},
 		{0}
 	};
 
 	if (ch->status & (STATUS_EMPTY | STATUS_MISSING)) {
-		rclick_menu[1].deactivate();
-		rclick_menu[5].deactivate();
-		rclick_menu[14].deactivate();
+		rclick_menu[(int) Menu::EXPORT_SAMPLE].deactivate();
+		rclick_menu[(int) Menu::EDIT_SAMPLE].deactivate();
+		rclick_menu[(int) Menu::FREE_CHANNEL].deactivate();
 	}
 
-	/* no 'clear actions' if there are no actions */
-
 	if (!ch->hasActions)
-		rclick_menu[7].deactivate();
+		rclick_menu[(int) Menu::CLEAR_ACTIONS].deactivate();
 
-	/* no 'clear start/stop actions' for those channels in loop mode:
-	 * they cannot have start/stop actions. */
+	/* No 'clear start/stop actions' for those channels in loop mode: they cannot
+  have start/stop actions. */
 
-	if (((SampleChannel*)ch)->mode & LOOP_ANY)
-		rclick_menu[11].deactivate();
+	if (static_cast<SampleChannel*>(ch)->mode & LOOP_ANY)
+		rclick_menu[(int) Menu::CLEAR_ACTIONS_START_STOP].deactivate();
 
 	Fl_Menu_Button *b = new Fl_Menu_Button(0, 0, 100, 50);
 	b->box(G_CUSTOM_BORDER_BOX);
@@ -191,130 +303,9 @@ void geSampleChannel::__cb_openMenu()
 	b->color(COLOR_BG_0);
 
 	const Fl_Menu_Item *m = rclick_menu->popup(Fl::event_x(), Fl::event_y(), 0, 0, b);
-	if (!m) return;
-
-	if (strcmp(m->label(), "Load new sample...") == 0) {
-    gWindow *w = new gdLoadBrowser(G_Conf.browserX, G_Conf.browserY,
-      G_Conf.browserW, G_Conf.browserH, "Browse sample",
-      G_Conf.samplePath.c_str(), glue_loadSample, ch);
-    gu_openSubWindow(G_MainWin, w, WID_FILE_BROWSER);
-		return;
-	}
-
-	if (strcmp(m->label(), "Setup keyboard input...") == 0) {
-		new gdKeyGrabber(ch); /// FIXME - use gu_openSubWindow
-		return;
-	}
-
-	if (strcmp(m->label(), "Setup MIDI input...") == 0) {
-		gu_openSubWindow(G_MainWin, new gdMidiInputChannel(ch), 0);
-		return;
-	}
-
-	if (strcmp(m->label(), "Setup MIDI output...") == 0) {
-		gu_openSubWindow(G_MainWin, new gdMidiOutputSampleCh((SampleChannel*) ch), 0);
-		return;
-	}
-
-	if (strcmp(m->label(), "Edit sample...") == 0) {
-		gu_openSubWindow(G_MainWin, new gdEditor((SampleChannel*) ch), WID_SAMPLE_EDITOR); /// FIXME title it's up to gdEditor
-		return;
-	}
-
-	if (strcmp(m->label(), "Export sample to file...") == 0) {
-    gWindow *w = new gdSaveBrowser(G_Conf.browserX, G_Conf.browserY,
-      G_Conf.browserW, G_Conf.browserH, "Save sample", \
-      G_Conf.samplePath.c_str(), "", glue_saveSample, ch);
-    gu_openSubWindow(G_MainWin, w, WID_FILE_BROWSER);
-		return;
-	}
-
-	if (strcmp(m->label(), "Delete channel") == 0) {
-		if (!gdConfirmWin("Warning", "Delete channel: are you sure?"))
-			return;
-		glue_deleteChannel(ch);
-		return;
-	}
-
-	if (strcmp(m->label(), "Free channel") == 0) {
-		if (ch->status == STATUS_PLAY) {
-			if (!gdConfirmWin("Warning", "This action will stop the channel: are you sure?"))
-				return;
-		}
-		else if (!gdConfirmWin("Warning", "Free channel: are you sure?"))
-			return;
-
-		glue_freeChannel(ch);
-
-		/* delete any related subwindow */
-
-		/** FIXME - use gu_closeAllSubwindows() */
-
-		G_MainWin->delSubWindow(WID_FILE_BROWSER);
-		G_MainWin->delSubWindow(WID_ACTION_EDITOR);
-		G_MainWin->delSubWindow(WID_SAMPLE_EDITOR);
-		G_MainWin->delSubWindow(WID_FX_LIST);
-
-		return;
-	}
-
-	if (strcmp(m->label(), "Clone channel") == 0) {
-		glue_cloneChannel(ch);
-		return;
-	}
-
-	if (strcmp(m->label(), "Mute") == 0) {
-		if (!gdConfirmWin("Warning", "Clear all mute actions: are you sure?"))
-			return;
-		G_Recorder.clearAction(ch->index, ACTION_MUTEON | ACTION_MUTEOFF);
-    ch->hasActions = G_Recorder.hasActions(ch->index);
-
-		if (!ch->hasActions)
-			hideActionButton();
-
-		/* TODO - set mute=false */
-
-		gu_refreshActionEditor(); // refresh a.editor window, it could be open
-		return;
-	}
-
-	if (strcmp(m->label(), "Start/Stop") == 0) {
-		if (!gdConfirmWin("Warning", "Clear all start/stop actions: are you sure?"))
-			return;
-		G_Recorder.clearAction(ch->index, ACTION_KEYPRESS | ACTION_KEYREL | ACTION_KILLCHAN);
-    ch->hasActions = G_Recorder.hasActions(ch->index);
-
-		if (!ch->hasActions)
-			hideActionButton();
-		gu_refreshActionEditor();  // refresh a.editor window, it could be open
-		return;
-	}
-
-	if (strcmp(m->label(), "Volume") == 0) {
-		if (!gdConfirmWin("Warning", "Clear all volume actions: are you sure?"))
-			return;
-		G_Recorder.clearAction(ch->index, ACTION_VOLUME);
-    ch->hasActions = G_Recorder.hasActions(ch->index);
-		if (!ch->hasActions)
-			hideActionButton();
-		gu_refreshActionEditor();  // refresh a.editor window, it could be open
-		return;
-	}
-
-	if (strcmp(m->label(), "All") == 0) {
-		if (!gdConfirmWin("Warning", "Clear all actions: are you sure?"))
-			return;
-		G_Recorder.clearChan(ch->index);
-    ch->hasActions = false;
-		hideActionButton();
-		gu_refreshActionEditor(); // refresh a.editor window, it could be open
-		return;
-	}
-
-	if (strcmp(m->label(), "Edit actions...") == 0) {
-		gu_openSubWindow(G_MainWin, new gdActionEditor(ch),	WID_ACTION_EDITOR);
-		return;
-	}
+  if (m)
+    m->do_callback(this, m->user_data());
+  return;
 }
 
 
@@ -337,11 +328,11 @@ void geSampleChannel::refresh()
 
 	setColorsByStatus(ch->status, ch->recStatus);
 
-	if (((SampleChannel*) ch)->wave != NULL) {
-		if (G_Mixer.recording && ch->armed)
+	if (((SampleChannel*) ch)->wave != nullptr) {
+		if (mixer::recording && ch->armed)
 			mainButton->setInputRecordMode();
-		if (G_Recorder.active) {
-			if (G_Recorder.canRec(ch, &G_Mixer))
+		if (recorder::active) {
+			if (recorder::canRec(ch, clock::isRunning(), mixer::recording))
 				mainButton->setActionRecordMode();
 		}
 		status->redraw(); // status invisible? sampleButton too (see below)
@@ -405,7 +396,7 @@ void geSampleChannel::update()
 	mainButton->setKey(ch->key);
 
 #ifdef WITH_VST
-	fx->full = ch->plugins.size() > 0;
+	fx->status = ch->plugins.size() > 0;
 	fx->redraw();
 #endif
 }
@@ -461,39 +452,3 @@ void geSampleChannel::resize(int X, int Y, int W, int H)
 
 	packWidgets();
 }
-
-
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-/* -------------------------------------------------------------------------- */
-
-
-geSampleChannelButton::geSampleChannelButton(int x, int y, int w, int h, const char *l)
-	: geChannelButton(x, y, w, h, l) {}
-
-
-/* -------------------------------------------------------------------------- */
-
-
-int geSampleChannelButton::handle(int e)
-{
-	int ret = gClick::handle(e);
-	switch (e) {
-		case FL_DND_ENTER:
-		case FL_DND_DRAG:
-		case FL_DND_RELEASE: {
-			ret = 1;
-			break;
-		}
-		case FL_PASTE: {
-      geSampleChannel *gch = (geSampleChannel*) parent();   // parent is geSampleChannel
-      SampleChannel   *ch  = (SampleChannel*) gch->ch;
-      int result = glue_loadChannel(ch, gu_trim(gu_stripFileUrl(Fl::event_text())).c_str());
-			if (result != SAMPLE_LOADED_OK)
-				G_MainWin->keyboard->printChannelMessage(result);
-			ret = 1;
-			break;
-		}
-	}
-	return ret;
-}
