@@ -6,7 +6,7 @@
  *
  * -----------------------------------------------------------------------------
  *
- * Copyright (C) 2010-2016 Giovanni A. Zuliani | Monocasual
+ * Copyright (C) 2010-2017 Giovanni A. Zuliani | Monocasual
  *
  * This file is part of Giada - Your Hardcore Loopmachine.
  *
@@ -27,6 +27,10 @@
  * -------------------------------------------------------------------------- */
 
 
+#ifndef G_UTILS_COCOA_H
+#define G_UTILS_COCOA_H
+
+
 /* fl_xid() from FLTK returns a pointer to NSWindow, but plugins on OS X want a
  * pointer to NSView. The function does the hard conversion. */
 
@@ -34,5 +38,8 @@ void *cocoa_getViewFromWindow(void *p);
 
 /* A bug on on OS X seems to misalign plugins' UI. The function takes care of
  * fixing the positioning. */
- 
+
 void cocoa_setWindowSize(void *p, int w, int h);
+
+
+#endif
